/*
 * Component:  utils
 *
 * Purpose: 
 *   Header file for semaphore implementation to get exclusive lock on a resource
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$ 
 * State            : $State$
 *
 */

#ifndef SEMAPHORE_H
#define SEMAPHORE_H

#include <string>
using namespace std;

/**
 * Creates/removes exclusive lock on a resource.
 */
class IMSemaphore {
	private:

		/** resource identifier */
		string m_resourceId;

		/** directory name where the lockfile should be created */
		string m_lockDir;

		/** indicates whether to release the lock when the semaphore object destroyed */
		bool m_releaseOnDestroy;

		/** indicates whether this semaphore locks the resource or not */
		bool m_isLocked;

		/** indicates whether unlock is forced even if the lock was created by another application */
		bool m_isUnlockForced;

		/**
		 * Attempts to lock the resource
		 *
		 * @return true if and only if the resource was successfully locked
		 *
		 * @throws IMIOException if semaphore file did not exist and could not be created.
		 */
		bool lockResource();

	public:

		/**
		 * Constructor for creating a new IMSemaphore object on a resource
		 *
		 * @param resourceId       identifies the resource to be semaphored
		 * @param lockDir          directory where lockfile will be created
		 *                         (if empty tries LOCK_VAR or if it is NULL then the current directory)
		 * @param releaseOnDestroy indicates whether a lock should be released when Semaphore is destroyed.
		 */
		IMSemaphore(const string & resourceId, const string & lockDir, bool releaseOnDestroy = true);

		/**
		 * Constructor for creating a new IMSemaphore object on a resource
		 *
		 * @param resourceId       identifies the resource to be semaphored
		 * @param releaseOnDestroy indicates whether a lock should be released when Semaphore is destroyed.
		 */
		IMSemaphore(const string & resourceId, bool releaseOnDestroy = true);

		/**
		 * Destructor to destroy IMSemaphor object
		 */
		~IMSemaphore();

		/**
		 * Tries to lock the resource. If the resource could not be locked, then lock will be retried
		 * several times until the timeout is expired.
		 *
		 * @param timeout   timeout in seconds (default is 30)
		 *
		 * @return true if and only if the resource was successfully locked
		 *
		 * @throws IMTimeoutException - if the timeout is reached
		 *         IMIOException if semaphore file did not exist and could not be created.
		 */
		bool lock( int timeout = 30 );

		/**
		 * Removes the lock created by this Semaphore object from the resource
		 *
		 * @throws IMIOException if semaphore file could not be released.
		 */
		void unlock();

		/**
		 * Checks if the resource is locked by this Semaphore object
		 *
		 * @return true if and only if this Semaphore object locks the resource.
		 */
		inline bool isLocked() {
			return m_isLocked;
		}

		/**
		 * Sets if the lock should be released on destroy
		 *
		 * @param releaseOnDestroy indicates whether a lock should be released when Semaphore is destroyed.
		 */
		inline void setReleaseOnDestroy( bool releaseOnDestroy ) {
			m_releaseOnDestroy = releaseOnDestroy;
		}

		/**
		 * Force unlocking semaphore, even if the lock was created by another application.
		 *
		 * TAKE EXTRA CARE WHEN USING THIS METHOD
		 */
		inline void forceUnlock() {
			m_isUnlockForced = true;
		}

};

#endif
